const express = require("express");
const multer = require("multer");
const path = require("path");
const voiceMessageController = require("../controllers/voiceMessageController");
const { protect } = require("../controllers/authController");

const router = express.Router();

// Set up storage for audio files
const storage = multer.diskStorage({
  destination: (req, file, cb) => {
    const uploadPath = path.join(__dirname, "..", "uploads", "audio");
    cb(null, uploadPath);
  },
  filename: (req, file, cb) => {
    const uniqueSuffix = Date.now() + "-" + Math.round(Math.random() * 1e9);
    cb(null, uniqueSuffix + "-" + file.originalname);
  },
});

// File filter to allow only specific audio file types
const fileFilter = (req, file, cb) => {
  const allowedExtensions = /mp3|aac|ogg|wav|amr|flac/;
  const isValid = allowedExtensions.test(
    path.extname(file.originalname).toLowerCase()
  );

  if (isValid) {
    cb(null, true);
  } else {
    cb(new Error("Invalid file type. Only audio files are allowed."));
  }
};

// Allow up to 5 audio files at once, with a max file size of 10MB each
const upload = multer({
  storage,
  fileFilter,
  limits: { fileSize: 10 * 1024 * 1024 }, // Max file size: 10MB
}).array("audio", 5); // Must match the Postman key name

// Route to send voice messages (receiverId in params)
router.use(protect);

// Route to send multiple voice messages (receiverId in params)
router.post(
  "/message/:receiverId",
  upload,
  voiceMessageController.sendVoiceMessage
);
module.exports = router;
