const Notification = require("../models/notificationModel");

const createAndEmitNotification = async (io, userId, message, type, link) => {
  try {
    // Validate type to ensure it's one of the allowed values
    const validTypes = ["connection", "like", "comment", "message", "other"];
    if (!validTypes.includes(type)) {
      throw new Error("Invalid notification type.");
    }

    // Create a new notification for the user
    const newNotification = new Notification({
      user: userId, // User who will receive the notification
      message: message,
      type: type, // Ensure this is one of the valid types
      link: link,
    });

    // Save the notification to the database
    await newNotification.save();

    // Emit the notification to the user's socket for real-time updates
    if (io && userId) {
      io.to(userId.toString()).emit("New Notification", {
        message: newNotification.message,
        createdAt: newNotification.createdAt,
        link: newNotification.link,
      });
      console.log(`Notification sent to user ${userId}`);
    } else {
      console.warn("Socket or userId is missing, notification not emitted.");
    }
  } catch (error) {
    console.error("Error creating and emitting notification:", error.message);
    // Optionally, log more details about the error, including the user and message
    console.error(
      `Failed to send notification to user ${userId} with message: ${message}`
    );
  }
};

module.exports = { createAndEmitNotification };
