const nodemailer = require("nodemailer");
const pug = require("pug");
const { htmlToText } = require("html-to-text");

module.exports = class Email {
  constructor(user, options = {}) {
    this.to = user.email;
    this.firstName = user.firstName || "User";
    this.url = options.url || null;
    this.OTP = options.OTP || null;
    this.user = user; // Full user object if needed
    this.from = `Ellade App <${
      process.env.EMAIL_FROM || "no-reply@example.com"
    }>`;
  }

  // Returns the transport to be used
  newTransport() {
    // Custom SMTP transport
  return nodemailer.createTransport({
          host: "smtp.gmail.com", // ✅ Correct hostname
          port: 587, // ✅ Correct port for TLS
          secure: false, // ✅ Must be false for TLS (use `true` for SSL on port 465)
          auth: {
            user: process.env.EMAIL_USER,
            pass: process.env.EMAIL_PASS,
          },
          tls: {
            rejectUnauthorized: false, // ✅ Allow self-signed certificates (if needed)
          },
        });

  }

  // Method to send email based on a template and subject
  async send(template, subject) {
    try {
      const html = pug.renderFile(
        `${__dirname}/../views/email/${template}.pug`,
        {
          firstName: this.firstName,
          url: this.url,
          OTP: this.OTP,
          user: this.user,
          subject,
        }
      );

      const mailOptions = {
        from: this.from,
        to: this.to,
        subject,
        html,
        text: htmlToText(html, { wordwrap: 130 }), // Convert HTML to plain text
      };

      // Send the email using the newTransport() method
      await this.newTransport().sendMail(mailOptions);
    } catch (err) {
      console.error("Error sending email:", err);
      throw new Error("There was an error sending the email.");
    }
  }

  // Send the welcome email to the user
  async sendWelcome() {
    await this.send("welcome", "Welcome to the Eallade App!");
  }

  // Send a password reset email to the user
  async sendPasswordReset() {
    await this.send(
      "passwordReset",
      "OTP for Verification (valid for only 10 minutes)"
    );
  }
};