const { Server } = require("socket.io");
const https = require("http");
const express = require("express");
const fs = require("fs");
// Paths to SSL/TLS certificates
const privateKeyPath = "path_to_private_key_file.pem";
const certificatePath = "path_to_certificate_file.pem";
const caPath = "path_to_ca_certificate_file.pem";

// Load SSL/TLS certificates
const privateKey = fs.readFileSync(privateKeyPath, "utf8");
const certificate = fs.readFileSync(certificatePath, "utf8");
const ca = fs.readFileSync(caPath, "utf8");

const credentials = {
  key: privateKey,
  cert: certificate,
  ca: ca,
};

const app = express();
// const httpsServer = https.createServer(app);
const server = https.createServer(app);

const io = new Server(server, {
  cors: {
    origin: ["http://localhost:3000"], // Ensure this is your backend port, NOT frontend
    methods: ["GET", "POST"],
  },
});

// Store user socket mappings
const userSocketMap = {};
const activeRooms = {};
/**
 * Get receiver's socket ID
 * @param {string} receiverId - The receiver's user ID
 * @returns {string | undefined} - Socket ID if online, undefined otherwise
 */
const getReceiverSocketId = (receiverId) => userSocketMap[receiverId];

// Function to get all online users
const getAllOnlineUsers = () => {
  return Object.keys(userSocketMap); // Return list of userIds
};
const getRoomName = (userId1, userId2) => {
  return `room:${[userId1, userId2].sort().join("-")}`;
};

io.on("connection", (socket) => {
  console.log(`🔌 User connected: ${socket.id}`);

  // Extract userId from handshake query
  const userId = socket.handshake.query.userId;
  if (userId && userId !== "undefined") {
    userSocketMap[userId] = socket.id;
    console.log(`✅ User mapped: ${userId} -> ${socket.id}`);
  }

  // Emit updated online users list
  io.emit("getOnlineUsers", getAllOnlineUsers());

  /**
   * Handle new messages
   */
  socket.on("newMessage", async (data) => {
    console.log("📩 Received message data:", data);
    const { message, senderId, receiverId } = data;
  
  });
  
//   // Function to get all online users
// const getAllOnlineUsers = () => {
//   return Object.keys(userSocketMap); // Return list of userIds
// };

  /**
   * Handle seen messages
   */
  // Handle the 'seenMessage' event
socket.on("messageSeen", async ({ senderId, receiverId, seen }) => {
    const response = await seenMessage({ body: { senderId, receiverId, seen } });

    if (response.status === true) {
        const senderSocketId = getReceiverSocketId(senderId); // Sender's socket ID

        if (senderSocketId) {
            io.to(senderSocketId).emit("messageSeen", {
                senderId,
                receiverId, 
                seen 
            });
        }
    } else {
        socket.emit("error", response.error);
    }
});

socket.on("joinRoom", ({ senderId, receiverId }) => {
    const roomName = getRoomName(senderId, receiverId);

    // Join the room
    socket.join(roomName);
    console.log(`🚪 User ${senderId} joined room: ${roomName}`);

    // Initialize the room if it doesn't exist
    if (!activeRooms[roomName]) {
      activeRooms[roomName] = { users: new Set() };
    }
    activeRooms[roomName].users.add(senderId);

    // Notify the room that a user has joined
    io.to(roomName).emit("userJoinedRoom", { roomName, userId: senderId });

    // Check if both users are in the room and mark messages as seen
    // checkAndMarkMessagesAsSeen(roomName);
  });


  // Handle the 'updateMessage' event
  socket.on("updateMessage", async ({ messageId, newText, userId }) => {
    const response = await updateMessage({
      body: { messageId, newText, userId },
    });
    if (response.status === 200) {
      io.to(getReceiverSocketId(userId)).emit("messageUpdated", {
        messageId,
        newText,
      });
      io.to(socket.id).emit("messageUpdated", { messageId, newText });
    } else {
      socket.emit("error", response.error);
    }
  });

  // Handle the 'deleteMessage' event
  socket.on("deleteMessage", async ({ messageId, userId }) => {
    const response = await deleteMessage({ body: { messageId, userId } });
    if (response.status === 200) {
      io.to(getReceiverSocketId(userId)).emit("messageDeleted", { messageId });
      io.to(socket.id).emit("messageDeleted", { messageId });
    } else {
      socket.emit("error", response.error);
    }
  });
  // Handle the 'typing' event
  socket.on("typing", async ({ userId, receiverId, typing }) => {
    const response = await typingStatus({
      body: { userId, receiverId, typing },
    });
    if (response.status === 200) {
      // Emit typing status to the receiver
      io.to(getReceiverSocketId(receiverId)).emit("typing", {
        userId,
        typing,
      });
    } else {
      socket.emit("error", response.error);
    }
  });
socket.on("leaveRoom", ({ senderId }) => {
    for (const roomName in activeRooms) {
        if (activeRooms[roomName].users.has(senderId)) {
            // Remove the user from the room
            activeRooms[roomName].users.delete(senderId);
            socket.leave(roomName);
            console.log(`🚪 User ${senderId} left room: ${roomName}`);

            // Notify remaining users
            io.to(roomName).emit("userLeftRoom", { roomName, userId: senderId });

            // Close the room immediately if either user leaves
            delete activeRooms[roomName];
            console.log(`🧹 Room ${roomName} closed because one user left.`);
          
        }
    }
});

  /**
   * Handle user disconnection
   */
 socket.on("disconnect", () => {
    console.log(`User disconnected: ${socket.id}`);

    if (userId && userSocketMap[userId]) {
      delete userSocketMap[userId];
      console.log(`🗑️ User removed: ${userId}`);
    }

    // Remove user from all rooms
    for (const roomName in activeRooms) {
      if (activeRooms[roomName].users.has(userId)) {
        activeRooms[roomName].users.delete(userId);
        console.log(`🚪 User ${userId} left room: ${roomName}`);

        // Notify the room that the user has left
        io.to(roomName).emit("userLeftRoom", { roomName, userId });

        // Clean up the room if it's empty
        if (activeRooms[roomName].users.size === 0) {
          delete activeRooms[roomName];
          console.log(`🧹 Room cleaned up: ${roomName}`);
        }
      }
    }

    // Emit updated online users list
    io.emit("getOnlineUsers", getAllOnlineUsers());
  });
});
module.exports = { app, io, server, getReceiverSocketId, getAllOnlineUsers,getRoomName ,activeRooms};
